#!/usr/bin/env python3
# encoding: utf-8

"""
Generating data by inserting and requesting files.

Based on: https://github.com/freenet/pyFreenet/blob/master/tutorial.py
"""

import fcp
import math
import uuid
import requests
import logging
import string
from random import random, randint, choice


def main():
    # Set up logging
    logger = logging.getLogger('generateTraffic')

    # Filehandler for logging
    fh = logging.FileHandler('/home/kees/experiments/logs/generateTraffic.log')

    # Formatting
    formatter = logging.Formatter('%(asctime)s;%(levelname)s; %(message)s')
    fh.setFormatter(formatter)

    # Add file handler to logger
    logger.addHandler(fh)

    # IP addresses for nodes
    ipAddrNodeA = "145.100.110.146"
    ipAddrNodeB = "145.100.110.147"
    ipAddrNodeC = "145.100.110.148"
    ipAddrNodeD = "145.100.110.149"
    ipAddrNodeE = "145.100.110.150"
    ipAddrNodeF = "145.100.110.151"
    ipAddrNodeG = "145.100.110.152"
    ipAddrNodeH = "145.100.110.153"

    # List of IP addresses
    ipAddresses = [ipAddrNodeA,
                   ipAddrNodeB,
                   ipAddrNodeC,
                   ipAddrNodeD,
                   ipAddrNodeE,
                   ipAddrNodeF,
                   ipAddrNodeG,
                   ipAddrNodeH]

    # Loop over nodes
    for ipAddr in ipAddresses:

        # Connect to node
        try:
            node = fcp.FCPNode(host=ipAddr)
        except:
            logger.error('Cannot connect to %s.', ipAddr)
            continue

        # Generate at between 1 and 10 ASCII blocks of 32 KiB for data
        val = (choice(string.ascii_letters) *
               1024 *
               32 *
               math.ceil(10 * random()))

        # Generate random uri
        uri = "KSK@" + uuid.uuid4().hex

        # Do an insert
        node.put(uri, data=val, mimetype="text/plain")

        # Log insert
        logger.info('Insert %s of %i bytes in %s', uri, len(val), ipAddr)

        # Request the inserted file from a random node
        randomIpAddr = ipAddresses[randint(0, len(ipAddresses)-1)]
        r = requests.get("http://{}:8888/{}".format(randomIpAddr, uri))

        # Log request
        logger.info('Request existing %s from %s status code: %i',
                    uri,
                    randomIpAddr,
                    r.status_code)

        # Request a non-existing file
        uri = "KSK@" + uuid.uuid4().hex
        r = requests.get("http://{}:8888/{}".format(ipAddr, uri))

        # Log request
        logger.info('Request non-existing %s from %s status code: %i',
                    uri,
                    ipAddr,
                    r.status_code)


if __name__ == "__main__":
    main()
